using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.Collections;
using System.Threading;
using Microsoft.AnalysisServices;

namespace AmoTraceSample
{
	public partial class AmoTraceForm : Form
	{
		Server server = null;
		BackgroundWorker backgroundWorker = null;
		TreeNode prevNode = null;
		ArrayList nodeStack = new ArrayList();
		enum AddNodeType
		{
			NodeBegin,
			NodeEnd,
			NodeInsert
		};
		enum ButtonState
		{
			None,
			Connect,
			Process
		}
		
		public AmoTraceForm()
		{
			InitializeComponent();
            // Until the connection is established, only the Connection button is enabled
			SetButtonsState(ButtonState.Connect);
            //Create and initialize the BackgroundWorker
			InitializeBackgroundWorker();
		}

		//Reset the state of the buttons.
		private void SetButtonsState(ButtonState state)
		{
			switch (state)
			{
				case ButtonState.Connect:
					processButton.Enabled = false;
					connectionButton.Enabled = true;
					break;
				case ButtonState.Process:
					connectionButton.Enabled = false;
					processButton.Enabled = true;
					break;
				case ButtonState.None:
					connectionButton.Enabled = false;
					processButton.Enabled = false;
					break;
			}
		}
		private void InitializeBackgroundWorker()
		{
			this.backgroundWorker = new BackgroundWorker();
			this.backgroundWorker.WorkerReportsProgress = false;
			this.backgroundWorker.WorkerSupportsCancellation = true;
			this.backgroundWorker.RunWorkerCompleted += OnCompleted;
			this.backgroundWorker.DoWork += OnDoWork;
		}

		//This function is called when the user clicks the Connection button 
		private void connectionButton_Click(object sender, EventArgs e)
		{
			this.processProgress.Value = 0;
			this.reportTree.Nodes.Clear();

			//Open a connection to the server.
			this.server = new Server();
			try
			{
				this.server.Connect("Datasource=localhost;");
				//Subscribe to the trace event.
				SubscribeToSessionTrace();
	
				//When the connection is opened, it is possible to start processing, but still impossible to cancel the process.
				SetButtonsState(ButtonState.Process);
			}
			catch (ConnectionException ex)
			{
				MessageBox.Show(ex.Message);
			}
	}

		//Set the event handler that will be called when the trace event is fired.
		//Start tracing.
		private void SubscribeToSessionTrace()
		{
			this.server.SessionTrace.OnEvent += new TraceEventHandler(sessionTrace_OnEvent);
			this.server.SessionTrace.Stopped += new TraceStoppedEventHandler(sessionTrace_Stopped);
			this.server.SessionTrace.Start(); // this method is not blocking
		}

		//Unsubscribe tracing events and stop tracing
		private void UnsubscribeFromSessionTrace()
		{
			this.server.SessionTrace.OnEvent -= new TraceEventHandler(sessionTrace_OnEvent);
			this.server.SessionTrace.Stopped -= new TraceStoppedEventHandler(sessionTrace_Stopped);
			this.server.SessionTrace.Stop();
		}

		// This function is called when the user clicks the Process button 	
		private void processButton_Click(object sender, EventArgs e)
		{
			// Get a database
			Database database = this.server.Databases.GetByName("Foodmart 2005");
			Cube cube = database.Cubes.GetByName("Warehouse and Sales");
			// Before starting the worker thread, disable the Process button so the user wont be able to click it again. 
			SetButtonsState(ButtonState.None);
			// Start the worker thread, then pass the Database object as a parameter to the function that will be called on the worker thread
			this.backgroundWorker.RunWorkerAsync(cube);
		}

		// This function is called on the background thread.
		// Process the cube
		void OnDoWork(object sender, DoWorkEventArgs doWorkArgs)
        {
			// Retrieve the Database object from the doWorkArgs parameter
			BackgroundWorker backgroundWorker;
			backgroundWorker = sender as BackgroundWorker;
			Cube cube = (Cube)doWorkArgs.Argument;
			//Start processing
			cube.Process(ProcessType.ProcessFull);
			doWorkArgs.Result = true;
		}

		//This function is called by the class BackgroundWorker, on the main thread when the function running in the background completes
		void OnCompleted(object sender, RunWorkerCompletedEventArgs completedArgs)
		{
			// If there was an error, notify the user.
			if(completedArgs.Error != null)
			{
				MessageBox.Show(completedArgs.Error.Message,completedArgs.Error.Source);
			}
			else
			{
				// The processing completed successfully
				MessageBox.Show(" Processing has been completed ");
			}
			//Execution of the command completed, close the connection and reset all buttons.
			if (null != this.server && this.server.Connected)
			{
				//unsibscribe the trace events, stop tracing
				if (this.server.SessionTrace != null)
				{
					// here we need to wait until retrieve trace events
					Thread.Sleep(500);
					UnsubscribeFromSessionTrace();
				}
				//Disconnect from the server.
				this.server.Disconnect();
			}		
			this.server = null;
			SetButtonsState(ButtonState.Connect);
		}

		/// The function will be called when Trace event is fired
		private void sessionTrace_OnEvent(object sender, TraceEventArgs e)
		{
			try
			{
				//Increment the progress bar
				if ( TraceEventClass.ProgressReportCurrent == e.EventClass )
					IncrementReport();

				//create new node on the report tree or update the existing one 
				switch (e.EventClass)
				{
					case TraceEventClass.ProgressReportBegin:
						AddNode(e.TextData.Trim(),AddNodeType.NodeBegin, Color.Black);
						return;
					case TraceEventClass.ProgressReportEnd:
						AddNode(e.TextData.Trim(), AddNodeType.NodeEnd, Color.Black);
						return;
					case TraceEventClass.ProgressReportError:
						AddNode(e.TextData.Trim(), AddNodeType.NodeInsert, Color.Red);
						break;
				}
			}
			catch (Exception ex)
			{
				MessageBox.Show(ex.Message);
			}
		}

		// Increments progress report bar.
		// Since Trace events are called from the thread created by AMO, thead marshaling is requered to access UI controls
		delegate void IncrementReportDelegate();
		void IncrementReport()
		{
			if (processProgress.InvokeRequired)
			{
				IncrementReportDelegate d = new IncrementReportDelegate(IncrementReport);
				this.Invoke(d);
			}
			else
			{
				processProgress.Increment(10);
			}
		}

		/// Add or update node of the report tree
		/// Since Trace events are called from the thread created by AMO, thead marshaling is requered to access UI controls
		delegate void AddNodeDelegate(string text, AddNodeType addNodeType, Color color);
		void AddNode(string text, AddNodeType addNodeType, Color color)
		{
			if (reportTree.InvokeRequired)
			{
				AddNodeDelegate d = new AddNodeDelegate(AddNode);
				this.Invoke(d, new object[] { text, addNodeType ,color});
			}
			else
			{
				TreeNode node;
				switch (addNodeType)
				{
					case AddNodeType.NodeBegin:
						node = new TreeNode(text);
						if (null != this.prevNode)
						{
							this.prevNode.Nodes.Add(node);
							this.prevNode.Expand();
						}
						else
							this.reportTree.Nodes.Add(node);
						this.prevNode = node;
						this.nodeStack.Add(node);
						break;
					case AddNodeType.NodeEnd:
						node = (TreeNode)this.nodeStack[this.nodeStack.Count - 1];
						node.Text = text;
						this.prevNode = node.Parent;
						this.nodeStack.Remove(node);
						break;
					case AddNodeType.NodeInsert:
						node = new TreeNode(text);
						if (null != this.prevNode)
							this.prevNode.Nodes.Add(node);
						else
							this.reportTree.Nodes.Add(node);
						this.prevNode = node;
						this.nodeStack.Add(node);
						break;
					default:
						node = null;
						break;
				}
				node.ForeColor = color;
			}
		}

		// This mehtod is called when trace is stoped.
		private void sessionTrace_Stopped(ITrace sender, TraceStoppedEventArgs e)
		{
			if (e.StopCause == TraceStopCause.StoppedByException)
			{

				if (e.Exception == null)
					MessageBox.Show("Trace has been stopped");
				else
					MessageBox.Show("Trace has been stopped with exception: " + e.Exception.Message );

			}
		}

	}
}